﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;

namespace XmlTree
{
    public partial class XmlForm : Form
    {
        public XmlForm()
        {
            InitializeComponent();
        }

        private void showStatus(string s)
        {
            statusLabel.Text = s;
        }

        private void showInfo (string s)
        {
            showStatus (s);
            // textBox.AppendText (s + "\r\n");
        }

        private void showProperties (object n)
        {
            propertyGrid.SelectedObject = n;
        }

        private void runButton_Click(object sender, EventArgs e)
        {
            // readSimpleXml("c:\\work\\data.xml");
            // openSimpleXml("c:\\work\\data.xml");
            readDom ("..\\..\\data.xml");
        }

        private void treeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode n = treeView.SelectedNode;
            MyTreeNode m = n as MyTreeNode;
            if (m != null)
                showProperties (m.obj);
        }

        private void showDom(XmlNode node, MyTreeNode top)
        {
            MyTreeNode n = new MyTreeNode();
            n.obj = node;
            if (node.NodeType == XmlNodeType.Text)
                n.Text = node.Value;
            else
                n.Text = "<" + node.Name + ">";
            if (top == null)
                treeView.Nodes.Add(n);
            else
                top.Nodes.Add(n);

            if (node.Attributes != null)
            {
                foreach (XmlAttribute attr in node.Attributes)
                {
                    MyTreeNode a = new MyTreeNode ();
                    a.obj = attr;
                    a.Text = attr.Name + " = " + attr.Value;
                    n.Nodes.Add (a);
                }
            }


            foreach (XmlNode child in node.ChildNodes)
                showDom (child, n);
        }


        private XmlTextReader g = null;

        private void openSimpleXml (string filePath)
        {
            g = new XmlTextReader(filePath);
            g.Read();
            showStatus ("First element");
            showInfo("Element: " + g.Name);
            showProperties(g);
        }

        private void nextButton_Click(object sender, EventArgs e)
        {
            g.Read ();
            showStatus ("Next element");
            showInfo("Element: " + g.Name);
            showProperties(g);
        }

        private void readSimpleXml(string filePath)
        {
            TreeNode current = null;

            // using System.Xml;
            XmlTextReader f = new XmlTextReader(filePath);
            while (f.Read())
            {
                if (f.NodeType == XmlNodeType.Element)
                {
                    TreeNode n = new TreeNode();
                    n.Text = f.Name;
                    if (current == null)
                        treeView.Nodes.Add(n);
                    else
                        current.Nodes.Add(n);
                    current = n;

                    if (f.HasAttributes)
                    {
                        f.MoveToFirstAttribute();
                        for (int i = 0; i < f.AttributeCount; i++)
                        {

                            TreeNode a = new TreeNode();
                            a.Text = "Attribute: " + f.Name + "=" + f.Value;
                            current.Nodes.Add(a);
                            f.MoveToNextAttribute();
                        }
                        f.MoveToElement();
                    }
                    if (f.IsEmptyElement)
                        current = current.Parent;
                }
                else if (f.NodeType == XmlNodeType.EndElement)
                {
                    current = current.Parent;
                }
                else if (f.NodeType == XmlNodeType.Text)
                {
                    TreeNode n = new TreeNode();
                    n.Text = f.Value;
                    current.Nodes.Add(n);
                }
            }
        }

        private void openButton_Click(object sender, EventArgs e)
        {
            if (openDialog.ShowDialog() == DialogResult.OK)
                readDom(openDialog.FileName);
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            // saveDialog.FileName
            if (saveDialog.ShowDialog() == DialogResult.OK)
                writeDom (saveDialog.FileName);

        }

        private XmlDocument doc = null;

        private void readDom(string filePath)
        {
            doc = new XmlDocument();
            doc.Load(filePath);
            showDom(doc.DocumentElement, null);
        }

        void writeDom(string fileName)
        {
            if (doc != null)
            {
                doc.Save (fileName);
            }
        }

        private void toTextButton_Click(object sender, EventArgs e)
        {
            if (doc != null)
            {
                // using System.IO;
                StringWriter f = new StringWriter ();
                doc.Save (f);
                textBox.Text = f.ToString();
            }
        }

        private void fromTextButton_Click(object sender, EventArgs e)
        {
            try
            {
                XmlDocument d = new XmlDocument();
                d.LoadXml(textBox.Text);
                doc = d;
                treeView.Nodes.Clear();
                showDom(d.DocumentElement, null);
            }
            catch (XmlException ex)
            {
                showInfo (ex.Message);
                int line = ex.LineNumber;
                int pos = 0;
                for (int i = 0; i < line; i++)
                    pos += textBox.Lines[i].Length;
                pos += ex.LinePosition;
                textBox.Select (pos, 1);
            }
        }


    }
}
